/*3:*/
#line 113 "eggdata.w"


#ifndef EGGDATA_HEADER_DEFINES
#define EGGDATA_HEADER_DEFINES
#include <math.h>        

#include <iostream> 
#include <iomanip> 
#include <fstream> 
#include <cstdlib> 
#include <exception> 
#include <algorithm> 
#include <stdexcept> 
#include <string> 
#include <vector> 
#include <map> 

#define HAVE_FDSTREAM_COMPATIBILITY
#ifdef HAVE_FDSTREAM_COMPATIBILITY
#include "fdstream.hpp"
#endif

using namespace std;

#include <ctype.h> 
#include <assert.h> 

#include "timedate.h"

/*8:*/
#line 183 "eggdata.w"


class csv_parser{
private:
string line_buffer;
int charpos;
bool pending_field;

public:
csv_parser(string linebuf= ""){
new_line(linebuf);
}

~csv_parser(){
}

void new_line(string linebuf){
line_buffer= linebuf;
set_scan_pointer(0);
}

int scan_pointer(void){
return charpos;
}

void set_scan_pointer(int charnum= 0){
charpos= charnum;
if(charnum==0){
pending_field= false;
}
}

bool next_field(string&f);

private:
static bool isOctal(char c){
return(((c)>='0')&&((c)<='7'));
}

bool chars_left(int n){
return((charpos+(n-1))<line_buffer.length());
}
};

/*:8*//*11:*/
#line 368 "eggdata.w"

template<class Sample> class generic_eggsummary{
public:



static const unsigned char MissingSample= 255;




static const unsigned char BadSample= 254;



static const unsigned char OutsideLimitsSample= 253;



static const unsigned char TimeShiftedOutSample= 252;



static const unsigned int sampleBytes= sizeof(Sample);








int samples_per_record;
int seconds_per_record;
int records_per_packet;









int trial_size;







int eggs_reporting;







systemtime start_time;
systemtime end_time;





unsigned int seconds_of_data;



unsigned int seconds_per_row;


bool pseudorandom_data;







vector<unsigned int> egg_number;

public:







map<unsigned int,unsigned int> egg_index;







Sample*egg_data;


Sample*egg_cell(int row,int egg_index){
return egg_data+(egg_index+(row*eggs_reporting));
}


void allocate_egg_data(void){
assert(egg_data==NULL);
egg_data= new Sample[eggs_reporting*(seconds_of_data/seconds_per_row)];
}


void free_egg_data(void){
if(egg_data!=NULL){
delete egg_data;
}
egg_data= NULL;
}

public:




generic_eggsummary(){
egg_data= NULL;
}

~generic_eggsummary(){
free_egg_data();
}


Sample get_egg_index(int row,int egg_index){
return*egg_cell(row,egg_index);
}


void set_egg_index(int row,int egg_index,Sample value){
#if 0
cout<<"Set egg index("<<row<<", "<<egg_index<<") = "<<((int)value)<<"\n";
#endif
*egg_cell(row,egg_index)= value;
}


int egg_number_to_index(int egg_number){
map<unsigned int,unsigned int> ::iterator mpair;

mpair= egg_index.find(egg_number);
if(mpair==egg_index.end()){
throw(invalid_argument("egg_number_to_index: Invalid egg number"));
}
return mpair->second;
}


Sample get_egg_number(int row,int egg_number){
return get_egg_index(row,egg_number_to_index(egg_number));
}



bool isSampleValid(Sample sample){
return sample<=trial_size;
}


bool isSampleValid(int row,int egg_number){
return isSampleValid(get_egg_number(row,egg_number));
}


void load_from_CSV(istream&i);
void load_from_CSV(string filename);


void save_to_CSV(ostream&o,bool interpret_missing= false);
void save_to_CSV(string fileName,bool interpret_missing= false);


void exclude_bad_data(istream&i);
void exclude_bad_data(string filename);


void limit_to_range(Sample minLimit= 0,Sample maxLimit= 255,ostream*os= 0);


void time_shift(int egg_index,int seconds);


void extract_time_range(generic_eggsummary<Sample> *es,systemtime begin,systemtime end,int sec_row= 1);


void extract_eggs(generic_eggsummary<Sample> *es,vector<unsigned int> &eggs);


void describe(ostream&os= cout);
};


typedef generic_eggsummary<unsigned char> eggsummary;

/*:11*//*12:*/
#line 581 "eggdata.w"

template<class Sample> void generic_eggsummary<Sample> ::load_from_CSV(istream&i){
string s;
int egg_row= 0;

free_egg_data();

while(getline(i,s)){
csv_parser p(s);
string f;
int i,m_type,s_type,arg;



if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("load_from_CSV: Invalid major type"));
}

m_type= atoi(f.c_str());

switch(m_type){
case 10:
/*16:*/
#line 730 "eggdata.w"

/*15:*/
#line 714 "eggdata.w"

if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("load_from_CSV: Invalid subtype"));
}
s_type= atoi(f.c_str());

if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("load_from_CSV: Invalid argument"));
}
arg= atoi(f.c_str());

/*:15*/
#line 731 "eggdata.w"
;

switch(s_type){
case 1:
samples_per_record= arg;
break;

case 2:
seconds_per_record= arg;
break;

case 3:
records_per_packet= arg;
break;

case 4:
trial_size= arg;
assert(trial_size<MissingSample);
break;
}

/*:16*/
#line 603 "eggdata.w"
;
break;

case 11:
/*17:*/
#line 757 "eggdata.w"

/*15:*/
#line 714 "eggdata.w"

if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("load_from_CSV: Invalid subtype"));
}
s_type= atoi(f.c_str());

if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("load_from_CSV: Invalid argument"));
}
arg= atoi(f.c_str());

/*:15*/
#line 758 "eggdata.w"
;

switch(s_type){
case 1:
eggs_reporting= arg;
if(!p.next_field(f)||(f.length()<1)){
throw(out_of_range("load_from_CSV: Invalid Eggs reporting header"));
}
pseudorandom_data= f.find("pseudorandom")!=string::npos;
break;

case 2:
start_time.set_time(arg);
break;

case 3:
end_time.set_time(arg);
break;

case 4:
seconds_of_data= arg;
break;
}

/*:17*/
#line 607 "eggdata.w"
;
break;

case 12:
/*18:*/
#line 787 "eggdata.w"




for(i= 0;i<2;i++){
if(!p.next_field(f)){
throw(out_of_range("load_from_CSV: Invalid type 12 record"));
}
}




for(i= 0;i<eggs_reporting;i++){
unsigned int eggno;

if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("load_from_CSV: Invalid type 12 egg table"));
}
eggno= atoi(f.c_str());

egg_number.push_back(eggno);

egg_index.insert(make_pair(eggno,i));
}

/*:18*/
#line 611 "eggdata.w"
;
break;

case 13:
/*19:*/
#line 820 "eggdata.w"


time_t rtime,xrtime;


if(egg_data==NULL){
seconds_per_row= seconds_per_record/samples_per_record;
egg_data= new Sample[eggs_reporting*(seconds_of_data/
seconds_per_row)];


xrtime= start_time.get_time();
egg_row= 0;
}

if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("load_from_CSV: Invalid time in type 13 record"));
}
rtime= atoi(f.c_str());
assert(rtime==xrtime);


xrtime+= seconds_per_row;

if(!p.next_field(f)){
throw(out_of_range("load_from_CSV: Invalid place holder in type 13 record"));
}





#ifdef CLEAN_BUT_SLOW
for(i= 0;i<eggs_reporting;i++){
unsigned char eggdat= MissingSample;

if(!p.next_field(f)){
throw(out_of_range("load_from_CSV: Missing egg data in type 13 record"));
}
if(f.length()> 0){
if(!isdigit(f[0])){
throw(out_of_range("load_from_CSV: Invalid egg data in type 13 record"));
}
eggdat= atoi(f.c_str());
}
set_egg_index(egg_row,i,eggdat);
}
#else












const char*cp= s.c_str();
Sample*ep= egg_cell(egg_row,0);



cp= strchr(strchr(strchr(cp,',')+1,',')+1,',')+1;
for(i= 0;i<eggs_reporting;i++){
unsigned char eggdat;

if(!isdigit(*cp)){
eggdat= MissingSample;
}else{
eggdat= (*cp++)-'0';
while(isdigit(*cp)){
eggdat= (eggdat*10)+((*cp++)-'0');
}
}
*ep++= eggdat;
cp++;
}
#endif

egg_row++;

/*:19*/
#line 615 "eggdata.w"
;
break;
}
}
#ifdef CSV_LOAD_DEBUG
cout<<"CSV load complete.\n";cout.flush();
#endif
}

/*:12*//*13:*/
#line 631 "eggdata.w"

template<class Sample> void generic_eggsummary<Sample> ::load_from_CSV(string filename){
#ifdef HAVE_FDSTREAM_COMPATIBILITY
fdistream iscc;
#endif
ifstream is;
FILE*ip= NULL;
/*14:*/
#line 679 "eggdata.w"


#if (defined HAVE_GUNZIP) || (defined HAVE_GZCAT) || (defined HAVE_GZIP)
#define COMPRESSED_FILES
static const char Compressed_file_type[]= ".gz";

static const char Uncompress_command[]= 
#if(defined HAVE_GUNZIP)
"gunzip -c"
#elif(defined HAVE_GZCAT)
"gzcat"
#elif(defined HAVE_GZIP)
"gzip -cd"
#endif
;
#elif (defined HAVE_ZCAT) || (defined HAVE_UNCOMPRESS) || (defined HAVE_COMPRESS)
#define COMPRESSED_FILES
static const char Compressed_file_type[]= ".Z";

static const char Uncompress_command[]= 
#if(defined HAVE_ZCAT)
"zcat"
#elif(defined HAVE_HAVE_UNCOMPRESS)
"uncompress -c"
#elif(defined HAVE_COMPRESS)
"compress -cd"
#endif
;
#endif

/*:14*/
#line 638 "eggdata.w"
;

#ifdef COMPRESSED_FILES
if(filename.rfind(Compressed_file_type)!=string::npos){
string cmd(Uncompress_command);
cmd+= ' '+filename;
ip= popen(cmd.c_str(),"r");
#ifdef HAVE_FDSTREAM_COMPATIBILITY
iscc.attach(fileno(ip));
#else
is.attach(fileno(ip));
#endif
}else{
#endif
is.open(filename.c_str(),ios::in);
#ifdef COMPRESSED_FILES
}
#endif
#ifdef HAVE_FDSTREAM_COMPATIBILITY
if(ip==NULL){
load_from_CSV(is);
}else{
load_from_CSV(iscc);
}
#else
load_from_CSV(is);
#endif
if(ip!=NULL){
pclose(ip);
}
}

/*:13*//*20:*/
#line 906 "eggdata.w"

template<class Sample> void generic_eggsummary<Sample> ::save_to_CSV(ostream&o,
bool interpret_missing){
int i;

/*21:*/
#line 927 "eggdata.w"

o<<"10,1,"<<samples_per_record<<",\"Samples per record\""<<endl;
o<<"10,2,"<<seconds_per_record<<",\"Seconds per record\""<<endl;
o<<"10,3,"<<records_per_packet<<",\"Records per packet\""<<endl;
o<<"10,4,"<<trial_size<<",\"Trial size\""<<endl;

o<<"11,1,"<<eggs_reporting<<",\"Eggs reporting\""<<endl;
o<<"11,2,"<<start_time.get_time()<<",\"Start time\","<<
start_time.dateToString()<<" "<<start_time.timeToString()<<endl;
o<<"11,3,"<<end_time.get_time()<<",\"End time\","<<
end_time.dateToString()<<" "<<end_time.timeToString()<<endl;
o<<"11,4,"<<seconds_of_data<<",\"Seconds of data\""<<endl;

/*:21*/
#line 911 "eggdata.w"
;
/*22:*/
#line 944 "eggdata.w"

o<<"12,\"gmtime\",";
for(i= 0;i<eggs_reporting;i++){
o<<","<<egg_number[i];
}
o<<endl;

/*:22*/
#line 912 "eggdata.w"
;
/*23:*/
#line 957 "eggdata.w"

int nrows= seconds_of_data/seconds_per_row;
systemtime now= start_time;

for(int n= 0;n<nrows;n++){
o<<"13,"<<now.get_time()<<",";
for(int e= 0;e<eggs_reporting;e++){
Sample s= get_egg_index(n,e);
o<<",";
if((s<=trial_size)||interpret_missing){
o<<((double)get_egg_index(n,e));
}
}
o<<endl;
now.set_time(now.get_time()+seconds_per_row);
}

/*:23*/
#line 913 "eggdata.w"
;
}

template<class Sample> void generic_eggsummary<Sample> ::save_to_CSV(string fileName,
bool interpret_missing){
ofstream of(fileName.c_str());

save_to_CSV(of,interpret_missing);
}

/*:20*//*24:*/
#line 984 "eggdata.w"

template<class Sample> void generic_eggsummary<Sample> ::exclude_bad_data(istream&i){
string s;
systemtime startTime,endTime;
int rottenEgg;

while(getline(i,s)){
if((s.length()> 0)&&(s[0]!=';')&&(s[0]!='#')){
csv_parser p(s);
string f;
int i,m_type,s_type,arg;

if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("exclude_bad_data: Invalid record type"));
}

m_type= atoi(f.c_str());

switch(m_type){
case 47:
/*25:*/
#line 1034 "eggdata.w"


if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("exclude_bad_data: Invalid start time in type 47 record"));
}
if(f.find('-')!=string::npos){
startTime.fromString(f);
}else{
startTime.set_time(atoi(f.c_str()));
}

if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("exclude_bad_data: Invalid end time in type 47 record"));
}
if(f.find('-')!=string::npos){
endTime.fromString(f);
}else{
endTime.set_time(atoi(f.c_str()));
}

if(endTime.get_time()<startTime.get_time()){
throw(out_of_range("exclude_bad_data: End time before start time in type 47 record"));
}


if(!p.next_field(f)||f.length()<1||!isdigit(f[0])){
throw(out_of_range("exclude_bad_data: Invalid egg number in type 47 record"));
}
rottenEgg= atoi(f.c_str());
assert(rottenEgg>=1);

#ifdef EXCLUDE_DEBUG
cout<<rottenEgg<<": "<<startTime.dateToString()<<" "<<startTime.timeToString()<<
" --> "<<startTime.dateToString()<<" "<<endTime.timeToString()<<endl;
#endif

/*26:*/
#line 1081 "eggdata.w"

if(!((endTime.get_time()<start_time.get_time())||
(startTime.get_time()> end_time.get_time()))){
map<unsigned int,unsigned int> ::iterator mpair;

mpair= egg_index.find(rottenEgg);
if(mpair!=egg_index.end()){
int egg_column= mpair->second;

systemtime rowTime(start_time.get_time());
time_t endt= end_time.get_time();

if(endTime.get_time()<endt){
endt= endTime.get_time();
}

int row= 0;
while(rowTime.get_time()<=endt){
if((rowTime.get_time()>=startTime.get_time())&&
(rowTime.get_time()<=endTime.get_time())){
set_egg_index(row,egg_column,BadSample);
#ifdef EXCLUDE_DEBUG
cout<<"Zapped egg "<<rottenEgg<<" at "<<
rowTime.dateToString()<<" "<<rowTime.timeToString()<<
" index "<<row<<endl;
#endif
}
rowTime.set_time(rowTime.get_time()+seconds_per_row);
row++;
}
}
#ifdef EXCLUDE_DEBUG
else{cout<<"No such egg."<<endl;}
#endif
}
#ifdef EXCLUDE_DEBUG
else{cout<<"Quick reject."<<endl;}
#endif

/*:26*/
#line 1070 "eggdata.w"
;

/*:25*/
#line 1004 "eggdata.w"
;
break;

default:
throw(out_of_range("exclude_bad_data: Invalid record type"));
break;

}
}
}
#ifdef EXCLUDE_DEBUG
cout<<"Exclusion of bad data complete.\n";cout.flush();
#endif
}

/*:24*//*27:*/
#line 1127 "eggdata.w"

template<class Sample> void generic_eggsummary<Sample> ::exclude_bad_data(string filename){
ifstream is;
#ifdef HAVE_FDSTREAM_COMPATIBILITY
fdistream iscc;
#endif
FILE*ip= NULL;
/*14:*/
#line 679 "eggdata.w"


#if (defined HAVE_GUNZIP) || (defined HAVE_GZCAT) || (defined HAVE_GZIP)
#define COMPRESSED_FILES
static const char Compressed_file_type[]= ".gz";

static const char Uncompress_command[]= 
#if(defined HAVE_GUNZIP)
"gunzip -c"
#elif(defined HAVE_GZCAT)
"gzcat"
#elif(defined HAVE_GZIP)
"gzip -cd"
#endif
;
#elif (defined HAVE_ZCAT) || (defined HAVE_UNCOMPRESS) || (defined HAVE_COMPRESS)
#define COMPRESSED_FILES
static const char Compressed_file_type[]= ".Z";

static const char Uncompress_command[]= 
#if(defined HAVE_ZCAT)
"zcat"
#elif(defined HAVE_HAVE_UNCOMPRESS)
"uncompress -c"
#elif(defined HAVE_COMPRESS)
"compress -cd"
#endif
;
#endif

/*:14*/
#line 1134 "eggdata.w"
;

#ifdef COMPRESSED_FILES
if(filename.rfind(Compressed_file_type)!=string::npos){
string cmd(Uncompress_command);
cmd+= ' '+filename;
ip= popen(cmd.c_str(),"r");
#ifdef HAVE_FDSTREAM_COMPATIBILITY
iscc.attach(fileno(ip));
#else
is.attach(fileno(ip));
#endif
}else{
#endif
is.open(filename.c_str(),ios::in);
#ifdef COMPRESSED_FILES
}
#endif
#ifdef HAVE_FDSTREAM_COMPATIBILITY
if(ip==NULL){
exclude_bad_data(is);
}else{
exclude_bad_data(iscc);
}
#else
exclude_bad_data(is);
#endif
if(ip!=NULL){
pclose(ip);
}
}

/*:27*//*28:*/
#line 1179 "eggdata.w"

template<class Sample> 
void generic_eggsummary<Sample> ::limit_to_range(Sample minLimit,Sample maxLimit,ostream*os){
for(int i= 0;i<eggs_reporting;i++){
int row= 0;
for(time_t t= start_time.get_time();t<=end_time.get_time();t+= seconds_per_row){
if(isSampleValid(get_egg_index(row,i))){
Sample was= get_egg_index(row,i);
if((was<minLimit)||(was> maxLimit)){
set_egg_index(row,i,OutsideLimitsSample);
if(os!=0){
*os<<"limit_to_range: zapping egg "<<egg_number[i]<<" at "<<
systemtime(t).dateToString()<<" "<<systemtime(t).timeToString()<<
" was "<<((unsigned int)was)<<endl;
}
}
}
row++;
}
}
}

/*:28*//*29:*/
#line 1233 "eggdata.w"

template<class Sample> void generic_eggsummary<Sample> ::time_shift(int egg_index,int seconds)
{
assert((egg_index>=0)&&(egg_index<eggs_reporting));
int rows_to_shift= seconds/seconds_per_row,
nrows= seconds_of_data/seconds_per_row;

if(rows_to_shift!=0){
if(abs(seconds)>=seconds_of_data){
/*32:*/
#line 1302 "eggdata.w"

Sample*cellp= egg_cell(0,egg_index);
for(int i= 0;i<nrows;i++){
*cellp= TimeShiftedOutSample;
cellp+= eggs_reporting;
}

/*:32*/
#line 1242 "eggdata.w"
;
}else{
if(rows_to_shift<0){
/*30:*/
#line 1258 "eggdata.w"

rows_to_shift= -rows_to_shift;
int i,ncopy= nrows-rows_to_shift;
Sample*dstp= egg_cell(0,egg_index),
*srcp= egg_cell(rows_to_shift,egg_index);
for(i= 0;i<ncopy;i++){
*dstp= *srcp;
srcp+= eggs_reporting;
dstp+= eggs_reporting;
}

dstp= egg_cell(ncopy,egg_index);
for(i= 0;i<rows_to_shift;i++){
*dstp= TimeShiftedOutSample;
dstp+= eggs_reporting;
}

/*:30*/
#line 1245 "eggdata.w"
;
}else{
/*31:*/
#line 1280 "eggdata.w"

int i,ncopy= nrows-rows_to_shift;
Sample*srcp= egg_cell((nrows-1)-rows_to_shift,egg_index),
*dstp= egg_cell((nrows-1),egg_index);
for(i= 0;i<ncopy;i++){
*dstp= *srcp;
srcp-= eggs_reporting;
dstp-= eggs_reporting;
}

dstp= egg_cell(0,egg_index);
for(i= 0;i<rows_to_shift;i++){
*dstp= TimeShiftedOutSample;
dstp+= eggs_reporting;
}

/*:31*/
#line 1247 "eggdata.w"
;
}
}
}
}

/*:29*//*33:*/
#line 1321 "eggdata.w"

template<class Sample> void
generic_eggsummary<Sample> ::extract_time_range(generic_eggsummary<Sample> *es,
systemtime begin,systemtime end,
int sec_row){

assert(sec_row==1);
assert(sec_row>=seconds_per_row);
assert((begin.get_time()>=start_time.get_time())&&
(end.get_time()<=end_time.get_time()));

/*34:*/
#line 1342 "eggdata.w"

#define Cf(x) es->x =  x
Cf(samples_per_record);
Cf(seconds_per_record);
Cf(records_per_packet);
Cf(trial_size);
Cf(eggs_reporting);
Cf(pseudorandom_data);
Cf(egg_number);
Cf(egg_index);
#undef Cf

/*:34*/
#line 1332 "eggdata.w"
;
/*35:*/
#line 1358 "eggdata.w"

es->start_time.set_time(begin.get_time());
es->end_time.set_time(end.get_time());
es->seconds_of_data= end.get_time()-begin.get_time();
es->seconds_per_row= sec_row;

/*:35*/
#line 1333 "eggdata.w"
;
/*36:*/
#line 1369 "eggdata.w"

es->free_egg_data();
es->egg_data= new Sample[eggs_reporting*(seconds_of_data/
seconds_per_row)];
Sample*dstp= es->egg_cell(0,0),
*srcp= egg_cell(((es->start_time.get_time()-start_time.get_time())/
seconds_per_row),0);
if(seconds_per_row==es->seconds_per_row){
/*37:*/
#line 1388 "eggdata.w"

memcpy(dstp,srcp,sampleBytes*eggs_reporting*(es->seconds_of_data/es->seconds_per_row));

/*:37*/
#line 1377 "eggdata.w"
;
}else{
/*38:*/
#line 1392 "eggdata.w"

assert(false);

/*:38*/
#line 1379 "eggdata.w"
;
}

/*:36*/
#line 1334 "eggdata.w"
;
}

/*:33*//*39:*/
#line 1397 "eggdata.w"

template<class Sample> void
generic_eggsummary<Sample> ::extract_eggs(generic_eggsummary<Sample> *es,
vector<unsigned int> &eggs){

/*40:*/
#line 1411 "eggdata.w"

#define Cf(x) es->x =  x
Cf(samples_per_record);
Cf(seconds_per_record);
Cf(records_per_packet);
Cf(trial_size);
Cf(start_time);
Cf(end_time);
Cf(seconds_of_data);
Cf(seconds_per_row);
Cf(pseudorandom_data);
#undef Cf

/*:40*/
#line 1402 "eggdata.w"
;
/*41:*/
#line 1436 "eggdata.w"

es->egg_index.clear();
es->egg_number.clear();
vector<unsigned int> ::iterator s;
for(s= eggs.begin();s!=eggs.end();s++){

if(egg_index.find(*s)!=egg_index.end()){
es->egg_number.push_back(*s);
}
}
es->eggs_reporting= es->egg_number.size();
int e;
/*59:*/
#line 1891 "eggdata.w"

sort(es->egg_number.begin(),es->egg_number.end());
es->egg_index.clear();
for(e= 0;e<es->eggs_reporting;e++){
es->egg_index.insert(make_pair(es->egg_number[e],e));
}

/*:59*/
#line 1448 "eggdata.w"
;

/*:41*/
#line 1403 "eggdata.w"
;
/*42:*/
#line 1458 "eggdata.w"

es->free_egg_data();
es->allocate_egg_data();
for(e= 0;e<es->eggs_reporting;e++){
Sample*dstp= es->egg_cell(0,e),
*srcp= egg_cell(0,egg_number_to_index(es->egg_number[e]));
int nrows= seconds_of_data/seconds_per_row;
for(int j= 0;j<nrows;j++){
*dstp= *srcp;
dstp+= es->eggs_reporting;
srcp+= eggs_reporting;
}
}

/*:42*/
#line 1404 "eggdata.w"
;
}

/*:39*//*43:*/
#line 1478 "eggdata.w"

template<class Sample> void generic_eggsummary<Sample> ::describe(ostream&os){
os<<"Egg summary:"<<endl<<
"  Start time: "<<start_time.dateToString()<<" "<<start_time.timeToString()<<
".  End time: "<<end_time.dateToString()<<" "<<end_time.timeToString();
if(pseudorandom_data){
os<<" (Pseudorandom data)";
}
os<<endl;
os<<"  Seconds of data: "<<seconds_of_data<<
"  Seconds per row: "<<seconds_per_row<<
"  Bytes per sample: "<<sampleBytes<<endl;
os<<"  Samples per record: "<<samples_per_record<<
"  Seconds per record: "<<seconds_per_record<<
"  Records per packet: "<<records_per_packet<<
"  Trial size: "<<trial_size<<endl;
os<<"  Eggs reporting: "<<eggs_reporting<<":";
for(int i= 0;i<eggs_reporting;i++){
if((i%10)==9){
os<<endl<<"                 ";
}
os<<" "<<egg_number[i];
}
os<<endl;
}

/*:43*//*44:*/
#line 1517 "eggdata.w"

class eggdatabases{
private:

map<string,string> cat_to_path;

public:

eggdatabases(){
}

void add_database(string dbname,string path){
cat_to_path.insert(make_pair(dbname,path));
}

string path(string database_name){
map<string,string> ::const_iterator p= cat_to_path.find(database_name);
if(p==cat_to_path.end()){
throw(out_of_range("eggdatabases::path: unknown database name"));
}
return p->second;
}

string database_file(systemtime t,string which= "gcp");

/*46:*/
#line 1561 "eggdata.w"

void set_Fourmilab_defaults(void){
add_database("gcp","/files/Server/Backup/egg/eggsummary");
add_database("pseudo","/files/Server/Backup/egg/pseudoeggsummary");
}

void set_noosphere_defaults(void){
add_database("gcp","/home/httpd/html/data/eggsummary");
add_database("pseudo","/home/httpd/html/data/pseudoeggsummary");
}

/*:46*//*47:*/
#line 1579 "eggdata.w"


#ifdef HOSTNAME
void set_local_defaults(void){
if(HOSTNAME=="hayek.lan.fourmilab.ch"){
set_Fourmilab_defaults();
}else if(HOSTNAME=="noosphere.princeton.edu"){
set_noosphere_defaults();
}else{
string em= "eggdatabases::set_local_defaults: unknown host name ";
em+= HOSTNAME;
throw(out_of_range(em.c_str()));
}
}
#endif

/*:47*/
#line 1542 "eggdata.w"
;
};


/*:44*//*48:*/
#line 1618 "eggdata.w"

template<class T> class generic_eggsummary_cache{
private:




map<string,T*> name_cache;

eggdatabases*eDB;
string eWhich;
string rottenEggDatabase;
int minSample,maxSample;

public:

void setEggDatabases(eggdatabases*ed= 0,string which= "gcp"){
eDB= ed;
eWhich= which;
}

void setRottenEgg(string rottenEgg= ""){
rottenEggDatabase= rottenEgg;
}

void setSampleLimits(int minSamp= -1,int maxSamp= 1024){
minSample= minSamp;
maxSample= maxSamp;
}

generic_eggsummary_cache(eggdatabases*ed= 0,
string rottenEgg= "",
int minSamp= -1,int maxSamp= 1024,
string which= "gcp"){
setEggDatabases(ed,which);
setRottenEgg(rottenEgg);
setSampleLimits(minSamp,maxSamp);
}

T*get(string path_name);
T*get_by_date(string date);
T*get_by_date(systemtime&t);
T*get_by_date(time_t tt);

~generic_eggsummary_cache(){
purge();
}

void purge(void);


void extract_time_range(T*es,systemtime begin,systemtime end,int sec_row= 1);
};

typedef generic_eggsummary_cache<eggsummary> eggsummary_cache;

/*:48*//*49:*/
#line 1680 "eggdata.w"

template<class T> T*generic_eggsummary_cache<T> ::get(string path_name){
typename map<string,T*> ::iterator item;

item= name_cache.find(path_name);
if(item==name_cache.end()){
T*es= new T();

es->load_from_CSV(path_name);
es->pseudorandom_data= eWhich=="pseudo";
/*51:*/
#line 1739 "eggdata.w"

if(rottenEggDatabase!=""){
es->exclude_bad_data(rottenEggDatabase);
}
if(minSample>=0){
es->limit_to_range(minSample,maxSample);
}

/*:51*/
#line 1690 "eggdata.w"
;
name_cache.insert(make_pair(path_name,es));
#ifdef CACHE_DEBUG
cout<<"Loaded "<<path_name<<" from file.\n";cout.flush();
#endif
return es;
}
#ifdef CACHE_DEBUG
cout<<"Returned "<<path_name<<" from cache.\n";cout.flush();
#endif
return item->second;
}

/*:49*//*50:*/
#line 1710 "eggdata.w"

template<class T> T*generic_eggsummary_cache<T> ::get_by_date(string date){
systemtime t;

t.fromString(date);
return get(eDB->database_file(t,eWhich));
}

template<class T> T*generic_eggsummary_cache<T> ::get_by_date(systemtime&t){
return get(eDB->database_file(t,eWhich));
}

template<class T> T*generic_eggsummary_cache<T> ::get_by_date(time_t tt){
systemtime t(tt);

return get(eDB->database_file(t,eWhich));
}

/*:50*//*52:*/
#line 1751 "eggdata.w"

template<class T> void generic_eggsummary_cache<T> ::purge(void){
typename map<string,T*> ::iterator item;


for(item= name_cache.begin();item!=name_cache.end();item++){
delete item->second;
}
name_cache.clear();
#ifdef CACHE_DEBUG
cout<<"Purged map.\n";cout.flush();
#endif
}

/*:52*//*53:*/
#line 1775 "eggdata.w"

#undef CETR_DEBUG
template<class T> void
generic_eggsummary_cache<T> ::extract_time_range(T*es,
systemtime begin,systemtime end,
int sec_row){

assert(sec_row==1);
assert((begin.get_time()<end.get_time()));

int e;

/*54:*/
#line 1797 "eggdata.w"

es->free_egg_data();
es->seconds_of_data= (end.get_time()-begin.get_time())+1;
es->start_time= begin;
es->end_time= end;
es->seconds_per_row= sec_row;
es->egg_index.clear();
es->egg_number.clear();
es->eggs_reporting= 0;

/*:54*/
#line 1787 "eggdata.w"
;
/*55:*/
#line 1823 "eggdata.w"

systemtime t(begin.midnight());
bool firstLaid= true;
do{
#ifdef CETR_DEBUG
cout<<"First pass loading "<<t.dateToString()<<endl;
#endif
T*ds= get_by_date(t);
if(firstLaid){
/*56:*/
#line 1846 "eggdata.w"

es->samples_per_record= ds->samples_per_record;
es->seconds_per_record= ds->seconds_per_record;
es->records_per_packet= ds->records_per_packet;
es->trial_size= ds->trial_size;
es->pseudorandom_data= ds->pseudorandom_data;

/*:56*/
#line 1832 "eggdata.w"
;
firstLaid= false;
}else{
/*57:*/
#line 1859 "eggdata.w"

if((es->samples_per_record!=ds->samples_per_record)||
(es->seconds_per_record!=ds->seconds_per_record)||
(es->records_per_packet!=ds->records_per_packet)||
(es->trial_size!=ds->trial_size)||
(es->pseudorandom_data!=ds->pseudorandom_data)
){
throw(out_of_range("generic_eggsummary_cache::extract_time_range: Parameters differ in days in extract"));
}

/*:57*/
#line 1835 "eggdata.w"
;
}
/*58:*/
#line 1874 "eggdata.w"

for(e= 0;e<ds->eggs_reporting;e++){
if(es->egg_index.find(ds->egg_number[e])==es->egg_index.end()){
es->egg_number.push_back(ds->egg_number[e]);
es->egg_index.insert(make_pair(ds->egg_number[e],es->eggs_reporting));
es->eggs_reporting++;
}
}

/*:58*/
#line 1837 "eggdata.w"
;
t.nextDay();
}while(t.get_time()<end.get_time());

/*:55*/
#line 1788 "eggdata.w"
;
/*59:*/
#line 1891 "eggdata.w"

sort(es->egg_number.begin(),es->egg_number.end());
es->egg_index.clear();
for(e= 0;e<es->eggs_reporting;e++){
es->egg_index.insert(make_pair(es->egg_number[e],e));
}

/*:59*/
#line 1789 "eggdata.w"
;
/*60:*/
#line 1904 "eggdata.w"

int egg_data_size= es->eggs_reporting*(es->seconds_of_data/
es->seconds_per_row);
es->allocate_egg_data();
for(e= 0;e<egg_data_size;e++){
es->egg_data[e]= es->MissingSample;
}

t.set_time(begin.midnight());
systemtime now= begin;
firstLaid= true;
int nr= 0;
do{
#ifdef CETR_DEBUG
cout<<"Second pass loading "<<t.dateToString()<<endl;
#endif
T*ds= get_by_date(t);
if(firstLaid){
e= begin.get_time()-ds->start_time.get_time();
assert((e>=0)&&(e<systemtime::SecondsPerDay));
firstLaid= false;
}else{
e= 0;
}
/*61:*/
#line 1941 "eggdata.w"

while((now.get_time()<=ds->end_time.get_time())&&
(now.get_time()<=es->end_time.get_time())){
for(int n= 0;n<ds->eggs_reporting;n++){
es->set_egg_index(nr,es->egg_number_to_index(ds->egg_number[n]),
ds->get_egg_index(e,n));
}
e++;
nr++;
now.set_time(now.get_time()+1);
}

/*:61*/
#line 1928 "eggdata.w"
;
t.nextDay();
}while(t.get_time()<end.get_time());

/*:60*/
#line 1790 "eggdata.w"
;
}

/*:53*//*62:*/
#line 1963 "eggdata.w"

class egg_properties{
public:
static const int unknownAltitude= -9999;
unsigned int eggNumber;
double latitude,longitude,altitude;
string location,REGtype,hostName;


void setFromCSV(string s);


void describe(ostream&os= cout);


void tabulate(ostream&os= cout);

bool isAltitudeKnown(void){
return altitude!=unknownAltitude;
}
};

/*:62*//*67:*/
#line 2128 "eggdata.w"

class egg_properties_database{
private:
map<unsigned int,egg_properties*> eggs;
vector<egg_properties*> eggv;

public:
void loadFromCSV(istream&i);
void loadFromCSV(string fileName);
void describe(ostream&o= cout);
void tabulate(ostream&o= cout);
unsigned int size(void);
egg_properties*find(unsigned int egg_number);
egg_properties*operator[](int n);
};

/*:67*/
#line 142 "eggdata.w"

#endif

/*:3*/
