/*3:*/
#line 72 "fourier.w"


#ifndef FOURIER_HEADER_DEFINES
#define FOURIER_HEADER_DEFINES
#include <math.h>        

#include <iostream> 
#include <exception> 
#include <stdexcept> 
#include <string> 
#include <vector> 
#include <algorithm> 
using namespace std;

#include <assert.h> 

/*6:*/
#line 111 "fourier.w"

static const int pwr_two[]= {
1,2,4,8,
16,32,64,128,
256,512,1024,2048,
4096,8192,16384,32768,
65536,131072,262144,524288,
1048576,2097152,4194304,8388608,
16777216,33554432,67108864,134217728,
268435456,536870912,1073741824
};


/*:6*//*7:*/
#line 129 "fourier.w"

class fourierTransform{
public:
typedef struct{
double r,i;
}complex;

/*:7*//*8:*/
#line 142 "fourier.w"

private:

complex*c;
int ln;

void buildtable(void);
/*:8*//*9:*/
#line 152 "fourier.w"

unsigned int getTransformSize(void){
return pwr_two[ln];
}

unsigned int getTransformSizePowerOfTwo(void){
return static_cast<unsigned int> (ln);
}

/*:9*//*10:*/
#line 168 "fourier.w"

public:
fourierTransform(unsigned int nsamples):c(NULL){
setTransformSize(nsamples);
}

fourierTransform():c(NULL){
}

void setTransformSize(unsigned int nsamples){
int p= 1;

for(ln= 0;p<nsamples;ln++){
p<<= 1;
}
if(pwr_two[ln]!=nsamples){
throw(out_of_range("fourierTransform::setTransformSize: data vector size is not a power of two"));
}
buildtable();
}

~fourierTransform(){
if(c!=NULL){
delete c;
}
}

/*:10*//*11:*/
#line 202 "fourier.w"

void bitreverse(complex*x);
void fft(complex*x);
void ifft(complex*x);

void transform(complex*x){
bitreverse(x);
fft(x);
}

void inverse_transform(complex*x){
bitreverse(x);
ifft(x);
}

void power_spectrum(const complex*x,double*p);
};

/*:11*/
#line 88 "fourier.w"

#endif

/*:3*/
